/*
   This is an example of a class that "has" an array. Objects
   of this class actually contain a reference to an array. The array
   object gets created by this class's constructor, so an array object
   is created as soon as an InfiniteArray object is created.

   This class encapsulates the idea of an "infinitely long" array. Of
   course, an array cannot be inifinitely long, so this object creates
   the illusion of infinite length by making the object look like a periodic
   array.

   This class is an example of what is called an "abstract data type". This
   class appears to be defining a new kind of data type, the type of an
   inifinitely long (periodic) array.
*/

public class InfiniteArray
{
   // An instance reference variable.
   private int[] hiddenArray;

   // Note: The following declaration can also be used (in which case
   //       the default constructor can be empty).
   // An instance reference variable with a default array object.
   //int[] hiddenArray = new int[100];


   /**
      Construct an InfiniteArray object that has period 100.
   */
   public InfiniteArray()
   {
      hiddenArray = new int[100];
   }


   /**
      Construct an InfiniteArray object that has period size.
   */
   public InfiniteArray(int size)
   {
      hiddenArray = new int[size];
   }


   /**
      Put a value in the InfiniteArray object at "location" i.
   */
   public void setEntry(int i, int value)
   {
      int j = i % hiddenArray.length;
      hiddenArray[j] = value;
   }


   /**
      Put a value in the InfiniteArray object at "location" i.
   */
   public void setEntry(long i, int value)
   {
      int j = (int)(i % (long)hiddenArray.length);
      hiddenArray[j] = value;
   }


   /**
      Get the value stored at "location" i.
   */
   public int getEntry(int i)
   {
      int j = i % hiddenArray.length;
      return hiddenArray[j];
   }


   /**
      Get the value stored at "location" i.
   */
   public int getEntry(long i)
   {
      int j = (int)(i % (long)hiddenArray.length);
      return hiddenArray[j];
   }

   /**
      Return the period of the infinitly long array.
   */
   public int getPeriod()
   {
      return hiddenArray.length;
   }

}//InfiniteArray